@@ -376,8 +376,8 @@ append_properties (GArray *properties, event_data *evdata)
 }
 
 static gboolean
-signal_is_needed (const gchar *klass, const gchar *major, const gchar *minor,
-                  GArray **properties)
+signal_is_needed (AtkObject *obj, const gchar *klass, const gchar *major,
+                  const gchar *minor, GArray **properties)
 {
   gchar *data [4];
   event_data *evdata;
@@ -403,7 +403,18 @@ signal_is_needed (const gchar *klass, const gchar *major, const gchar *minor,
         !g_strcmp0 (data [2], "accessible-parent") ||
         !g_strcmp0 (data [2], "accessible-role"))) ||
       !g_strcmp0 (data [1], "StateChanged"))
-    ret = TRUE;
+  {
+    if (minor && !g_strcmp0 (minor, "defunct"))
+      ret = TRUE;
+    else
+    {
+      AtkStateSet *set = atk_object_ref_state_set (obj);
+      AtkState state = ((!g_strcmp0 (data[1], "ChildrenChanged")) ?
+                        ATK_STATE_MANAGES_DESCENDANTS : ATK_STATE_TRANSIENT);
+      ret = !atk_state_set_contains_state (set, state);
+      g_object_unref (set);
+    }
+  }
 
   /* Hack: events such as "object::text-changed::insert:system" as
      generated by Gecko */
@@ -483,7 +494,7 @@ emit_event (AtkObject  *obj,
   if (!minor) minor = "";
   if (!type) type = "u";
 
-  if (!signal_is_needed (klass, major, minor, &properties))
+  if (!signal_is_needed (obj, klass, major, minor, &properties))
     return;
 
   path =  spi_register_object_to_path (spi_global_register, G_OBJECT (obj));
